#!/usr/bin/env python
# -*- coding: latin-1 -*-

"""Interface into the Face-Tools Database

This provides  two functions,  which return either  all images  (for a
given  tag, or  all tags),  or all  tags (for  a given  image,  or all
images), iterator-style.
"""

__author__ = "Sven Utcke <sven.utcke@svenutcke.de"
__source__ = "http://www.svenutcke.de/sw/>"
__version__ = "$Revision: 1.2 $"
__license__ = """Copyright (C) 2011 Sven Utcke <sven.utcke@svenutcke.de>.

A  nonexclusive  license is  granted  to  use  this file  for  private
purposes.  The right to create derivative works is explicitly granted,
I would however recommend  to use python's 'import' statement instead.
A seperate license is required for comercial use.

There is NO WARRANTY, to the extent permitted by law.
"""

import sqlite3

def Tags(connection, image=None):
    """Iterate over all tags

    Only iterates over the tags connected with image, if specified.
    """
    cursor = connection.cursor()
    if not image:
        cursor.execute('''SELECT tag FROM Tags WHERE id>1''')
    else:
        cursor.execute('''SELECT Tags.tag FROM Images, Faces, Tags WHERE Images.path=? AND Images.id=Faces.imageid AND Faces.tagid=Tags.id AND Tags.id>1''',
                       [image,])
    for row in cursor:
        yield row[0]

def Images(connection, tag=None, tagged=True):
    """Iterate over all (tagged) images

    Only iterate over the images tagged as 'tag', if specified.  
    By default, return all tagged images (except for images tagged unknown).
    """
    cursor = connection.cursor()
    if not tag:
        if tagged:
            # select all tagged images
            cursor.execute('''SELECT Images.path FROM Images, Faces WHERE Faces.tagid > 1 AND Faces.imageid=Images.id GROUP BY Images.id''')
        else:
            # select _all_ images
            cursor.execute('''SELECT path FROM Images''')
    else:
        # all images with tag tag
        cursor.execute('''SELECT Images.path FROM Images, Faces, Tags WHERE Tags.tag=? AND Faces.tagid=Tags.id AND Images.id=Faces.imageid GROUP BY Images.id''',
                       [tag,])
    for row in cursor:
        yield row[0]


if __name__ == "__main__":
    """Example application

    This is a very simple example application, which simply produces
    commandlines which, when executed, will append tags to files.
    """
    import os.path
    db_file = os.path.abspath(os.path.expanduser(os.path.expandvars("$HOME/.face/faces.sql3")))
    conn = sqlite3.connect(db_file)

    for im in Images(conn):
        print "exiftool -overwrite_original",
        for tag in Tags(conn, im):
            # a) exiftool defaults to utf-8
            # b) first remove the keyword, then add it back in again, to avoid duplicates
            #    Based on an idea by Peter Hoener
            print '-Keywords-="' + tag.encode('utf-8') + '"', 
            print '-Keywords+="' + tag.encode('utf-8') + '"', 
        print im


